function ExtractBackground(dataset,nFrames,ifOtherDatasets)
%Time-average the movement regions in a video as the first stage of a
%trajectory-learning algorithm
%Alexander Farley
%alexander.farley at utoronto.ca
%September 16 2011
%Written and tested in Matlab R2011a
%------------------------------------------------------------------------------
%The purpose of this script is to average a video across all frames as a
%first stage of a trajectory-learning algorithm
if ~exist('ifOtherDatasets','var')
    ifOtherDatasets = 0;
end

switch dataset
    case {'boxx','doll','hous','pens','stnd','jto2','jtoy','hero'}
        thresh = 0.07;
    case {'elph','race'}
        thresh = 0.07;
end

bk_downsample = 1; %The downsample factor for frame averaging
if ifOtherDatasets
    decVal =4;
else
    decVal =3; 
end
oneStr = getImgnoInStrOrInt(1,decVal);
framePath1=sprintf('../images/%s/img%s.ppm',dataset,oneStr);
frame = imread(framePath1);

if 1
    %% First-iteration background frame
    [vidHeight vidWidth dummy] = size(frame);
    background_frame = zeros(vidHeight,vidWidth,3);
    
    for k = 1:bk_downsample:nFrames
        kStr = getImgnoInStrOrInt(k,decVal);
        framePath=sprintf('../images/%s/img%s.ppm',dataset,kStr);
        frame = imread(framePath);
        background_frame = background_frame + double(frame);
        %     disp(k/(nFrames)*100)
    end
    
    %background_frame = uint8(bk_downsample*background_frame/(nFrames));
    background_frame = bk_downsample*background_frame/(nFrames);
else
    background_frame = imread(sprintf('../bg/%s/imgB1.ppm',dataset));
%     background_frame = GetBGFromVideo(dataset,nFrames);
    if strcmp(dataset,'elph')
%        GetBetterBG('elph'); 
    end
end
%imshow(background_frame)

%% Second-iteration background frame
%This section re-calculates the background frame while attempting to
%minimize the effect of moving objects in the calculation

background_frame2 = double(frame*0);
pixel_sample_density = im2bw(double(frame*0));
diff_frame = double(frame*0);
stream_frame = diff_frame(:,:,1);

% figure, imshow(uint8(background_frame))

% hold on
for k = 1:bk_downsample:nFrames
    kStr = getImgnoInStrOrInt(k,decVal);
    framePath=sprintf('../images/%s/img%s.ppm',dataset,kStr);
    frame = imread(framePath);
    diff_frame = imabsdiff(double(frame), double(background_frame));
    diff_frame = 1-im2bw(uint8(diff_frame),thresh); %0.07 for jtoy
    pixel_sample_density = pixel_sample_density + diff_frame;
    stream_frame = stream_frame + (1-diff_frame)/(nFrames/bk_downsample);
    nonmoving = double(imread(framePath));
    nonmoving(:,:,1) = nonmoving(:,:,1).*diff_frame;
    nonmoving(:,:,2) = nonmoving(:,:,2).*diff_frame;
    nonmoving(:,:,3) = nonmoving(:,:,3).*diff_frame;
    %     figure, imshow(uint8(nonmoving))
    %     figure, imshow((diff_frame))
    background_frame2 = background_frame2 + nonmoving;
    %pause
    %     disp(k/(nFrames)*100)
end

background_frame2(:,:,1) = background_frame2(:,:,1)./pixel_sample_density;
background_frame2(:,:,2) = background_frame2(:,:,2)./pixel_sample_density;
background_frame2(:,:,3) = background_frame2(:,:,3)./pixel_sample_density;


figure, imshow(uint8(background_frame2));
imwrite(uint8(background_frame2),sprintf('../bg/%s/imgB.ppm',dataset));
if 0
    for k = 1:bk_downsample:nFrames
        kStr = getImgnoInStrOrInt(k,decVal);
        framePath=sprintf('../images/%s/img%s.ppm',dataset,kStr);
        frame = imread(framePath);
        diff_frame = imabsdiff(double(frame), background_frame);
        diff_frame = 1-im2bw(uint8(diff_frame),.07);
        pixel_sample_density = pixel_sample_density + diff_frame;
        stream_frame = stream_frame + (1-diff_frame)/(nFrames/bk_downsample);
        nonmoving = double(imread(framePath));
        nonmoving(:,:,1) = nonmoving(:,:,1).*diff_frame;
        nonmoving(:,:,2) = nonmoving(:,:,2).*diff_frame;
        nonmoving(:,:,3) = nonmoving(:,:,3).*diff_frame;
        figure, imshow(uint8(nonmoving))
        figure, imshow((diff_frame))
        background_frame2 = background_frame2 + nonmoving;
        %pause
        disp(k/(nFrames)*100)
    end
end

%imshow(stream_frame)


