#if !defined(DIJKSTRA_H)
#include "dijkstra.h" 
#endif

// A utility function to create a new adjacency list node
struct AdjListNode* newAdjListNode(int dest, float weight)
{
    struct AdjListNode* newNode =
            (struct AdjListNode*) malloc(sizeof(struct AdjListNode));
    newNode->dest = dest;
    newNode->weight = weight;
    newNode->next = NULL;
    return newNode;
}

// A utility function that creates a graph of V vertices
struct Graph* createGraph(int V)
{
    struct Graph* graph = (struct Graph*) malloc(sizeof(struct Graph));
    graph->V = V;
 
    // Create an array of adjacency lists.  Size of array will be V
    graph->array = (struct AdjList*) malloc(V * sizeof(struct AdjList));
 
     // Initialize each adjacency list as empty by making head as NULL
    for (int i = 0; i < V; ++i)
        graph->array[i].head = NULL;
 
    return graph;
}
 
// Adds an edge to an undirected graph
void addEdge(struct Graph* graph, int src, int dest, float weight)
{
    // Add an edge from src to dest.  A new node is added to the adjacency
    // list of src.  The node is added at the begining
    struct AdjListNode* newNode = newAdjListNode(dest, weight);
    newNode->next = graph->array[src].head;
    graph->array[src].head = newNode;
 
    // Since graph is undirected, add an edge from dest to src also
    newNode = newAdjListNode(src, weight);
    newNode->next = graph->array[dest].head;
    graph->array[dest].head = newNode;
}
 
// A utility function to create a new Min Heap Node
struct MinHeapNode* newMinHeapNode(int v, int dist)
{
    struct MinHeapNode* minHeapNode =
           (struct MinHeapNode*) malloc(sizeof(struct MinHeapNode));
    minHeapNode->v = v;
    minHeapNode->dist = dist;
    return minHeapNode;
}
 
// A utility function to create a Min Heap
struct MinHeap* createMinHeap(int capacity)
{
    struct MinHeap* minHeap =
         (struct MinHeap*) malloc(sizeof(struct MinHeap));
    minHeap->pos = (int *)malloc(capacity * sizeof(int));
    minHeap->size = 0;
    minHeap->capacity = capacity;
    minHeap->array =
         (struct MinHeapNode**) malloc(capacity * sizeof(struct MinHeapNode*));
    return minHeap;
}
 
// A utility function to swap two nodes of min heap. Needed for min heapify
void swapMinHeapNode(struct MinHeapNode** a, struct MinHeapNode** b)
{
    struct MinHeapNode* t = *a;
    *a = *b;
    *b = t;
}
 
// A standard function to heapify at given idx
// This function also updates position of nodes when they are swapped.
// Position is needed for decreaseKey()
void minHeapify(struct MinHeap* minHeap, int idx)
{
    int smallest, left, right;
    smallest = idx;
    left = 2 * idx + 1;
    right = 2 * idx + 2;
 
    if (left < minHeap->size &&
        minHeap->array[left]->dist < minHeap->array[smallest]->dist )
      smallest = left;
 
    if (right < minHeap->size &&
        minHeap->array[right]->dist < minHeap->array[smallest]->dist )
      smallest = right;
 
    if (smallest != idx)
    {
        // The nodes to be swapped in min heap
        MinHeapNode *smallestNode = minHeap->array[smallest];
        MinHeapNode *idxNode = minHeap->array[idx];
 
        // Swap positions
        minHeap->pos[smallestNode->v] = idx;
        minHeap->pos[idxNode->v] = smallest;
 
        // Swap nodes
        swapMinHeapNode(&minHeap->array[smallest], &minHeap->array[idx]);
 
        minHeapify(minHeap, smallest);
    }
}
 
// A utility function to check if the given minHeap is ampty or not
int isEmpty(struct MinHeap* minHeap)
{
    return minHeap->size == 0;
}
 
// Standard function to extract minimum node from heap
struct MinHeapNode* extractMin(struct MinHeap* minHeap)
{
    if (isEmpty(minHeap))
        return NULL;
 
    // Store the root node
    struct MinHeapNode* root = minHeap->array[0];
 
    // Replace root node with last node
    struct MinHeapNode* lastNode = minHeap->array[minHeap->size - 1];
    minHeap->array[0] = lastNode;
 
    // Update position of last node
    minHeap->pos[root->v] = minHeap->size-1;
    minHeap->pos[lastNode->v] = 0;
 
    // Reduce heap size and heapify root
    --minHeap->size;
    minHeapify(minHeap, 0);
 
    return root;
}
 
// Function to decreasy dist value of a given vertex v. This function
// uses pos[] of min heap to get the current index of node in min heap
void decreaseKey(struct MinHeap* minHeap, int v, float dist)
{
    // Get the index of v in  heap array
    int i = minHeap->pos[v];
 
    // Get the node and update its dist value
    minHeap->array[i]->dist = dist;
 
    // Travel up while the complete tree is not hepified.
    // This is a O(Logn) loop
    while (i && minHeap->array[i]->dist < minHeap->array[(i - 1) / 2]->dist)
    {
        // Swap this node with its parent
        minHeap->pos[minHeap->array[i]->v] = (i-1)/2;
        minHeap->pos[minHeap->array[(i-1)/2]->v] = i;
        swapMinHeapNode(&minHeap->array[i],  &minHeap->array[(i - 1) / 2]);
 
        // move to parent index
        i = (i - 1) / 2;
    }
}
 
// A utility function to check if a given vertex
// 'v' is in min heap or not
bool isInMinHeap(struct MinHeap *minHeap, int v)
{
   if (minHeap->pos[v] < minHeap->size)
     return true;
   return false;
}
 
// A utility function used to print the solution
void printArr(float dist[], int n, map<int, myPoint> valToPoint, vector<int> prev, int src, int dest)
{
	//printf("Vertex   Distance from Source\n");
    for (int i = 0; i < n; ++i)
	{
        //printf("(%d,%d) \t\t %f\n", valToPoint[i].x, valToPoint[i].y, dist[i]);
	}

	//printf("\nShortest path\n");
	int i = dest;
	int counts = 0;
	while(prev[i] != i)
	{
		counts ++;
		i = prev[i];
	}
	counts ++;
	cout << counts << endl;
	i = dest;
	//Mat img(4000, 4000, CV_8UC3, Scalar(255, 255, 255));
	while(prev[i] != i)
	{
		valToPoint[i].display();
		myPoint temp = valToPoint[i];
		//circle(img, Point(temp.x, temp.y), 2, Scalar(255, 0, 0), 1);
		i = prev[i];
	}
	
	//resize(img, img, Size(), 0.5, 0.5);
	//namedWindow("Image", CV_WINDOW_NORMAL);
	//circle(img, Point(2300, 1200), 10, Scalar(0, 255, 0), 10);
	//imshow("Image", img(Rect(2000, 1000, 1000, 1000)));
	//waitKey(0);
	valToPoint[src].display();
}
 
// The main function that calulates distances of shortest paths from src to all
// vertices. It is a O(ELogV) function
void dijkstra(struct Graph* graph, int src, int dest, map<int, myPoint> valToPoint, vector<myPoint> outputPoints)
{
    int V = graph->V;// Get the number of vertices in graph
    float dist[V];      // dist values used to pick minimum weight edge in cut
	vector<int> prev;
    // minHeap represents set E
    struct MinHeap* minHeap = createMinHeap(V);
 
    // Initialize min heap with all vertices. dist value of all vertices 
    for (int v = 0; v < V; ++v)
    {
        dist[v] = INF;
        minHeap->array[v] = newMinHeapNode(v, dist[v]);
        minHeap->pos[v] = v;
		prev.push_back(v);
    }
 
    // Make dist value of src vertex as 0 so that it is extracted first
    minHeap->array[src] = newMinHeapNode(src, dist[src]);
    minHeap->pos[src]   = src;
    dist[src] = 0;
    decreaseKey(minHeap, src, dist[src]);

    // Initially size of min heap is equal to V
    minHeap->size = V;
 
    // In the followin loop, min heap contains all nodes
    // whose shortest distance is not yet finalized.
    while (!isEmpty(minHeap))
    {
        // Extract the vertex with minimum distance value
        struct MinHeapNode* minHeapNode = extractMin(minHeap);
        int u = minHeapNode->v; // Store the extracted vertex number
        // Traverse through all adjacent vertices of u (the extracted
        // vertex) and update their distance values
        struct AdjListNode* pCrawl = graph->array[u].head;
        while (pCrawl != NULL)
        {
            int v = pCrawl->dest;
 
            // If shortest distance to v is not finalized yet, and distance to v
            // through u is less than its previously calculated distance
            if (isInMinHeap(minHeap, v) && dist[u] != INF && 
                                          pCrawl->weight + dist[u] < dist[v])
            {
                dist[v] = dist[u] + pCrawl->weight;
				prev[v] = u; 
                // update distance value in min heap also
                decreaseKey(minHeap, v, dist[v]);
            }
            pCrawl = pCrawl->next;
        }
    }
	
	int i = dest;
	int counts = 0;

	while(prev[i] != i)
	{
		counts ++;
		i = prev[i];
	}
	counts ++;
	i = dest;
	//Mat img(4000, 4000, CV_8UC3, Scalar(255, 255, 255));
	while(prev[i] != i)
	{
		//valToPoint[i].display();
		myPoint temp = valToPoint[i];
		outputPoints.push_back(temp);
		//circle(img, Point(temp.x, temp.y), 2, Scalar(255, 0, 0), 1);
		i = prev[i];
	}

	outputPoints.push_back(valToPoint[src]);

    // print the calculated shortest distances
    //printArr(dist, V, valToPoint, prev, src, dest);
}

Graph* assignToGraph(vector<myPoint> points, map<int,list<int> > dicto, list<int> y_list, map<int, myPoint> &valToPoint, map<myPoint, int> &pointToVal, myPoint &src_pt, myPoint &dst_pt)
{
	for(int i=0; i<points.size(); i++)
	{
		valToPoint[i] = points[i];
		pointToVal[points[i]] = i;
	}

	int V = points.size();
	struct Graph* graph = createGraph(V);
				
	myPoint p1, p2;
	//myPoint src_pt(2370, 1377), dst_pt(2369, 1378);
	
	bool found = false;

	for(list<int>::iterator it = y_list.begin(); it != y_list.end(); ++it)
	{
		for(list<int>::iterator it2 = dicto[*it].begin(); it2 != dicto[*it].end(); ++it2)
		{
			p1 = myPoint(*it2, *it);
			list<int>::iterator it3 = it2;
			++it3;

			for(; it3 != dicto[*it].end(); ++it3)
			{
				p2 = myPoint(*it3, *it);
				if(chessDistance(p1, p2) == 1)
				{
					src_pt = p1;
					dst_pt = p2;
					found = true;
					break;
				}
			}
			
			list<int>::iterator it_temp = y_list.end();
			--it_temp;
			if(it != it_temp && !found )
			{
				list<int>::iterator it2_temp = it;
				++it2_temp;
				for(it3 = dicto[*(it2_temp)].begin(); it3 != dicto[*(it2_temp)].end(); ++it3)
				{
					p2 = myPoint(*it3, *(it2_temp));
					if(chessDistance(p1, p2) == 1)
					{
						found = true;
						src_pt = p1;
						dst_pt = p2;
						break;
					}	
				}
			}
			if(found)
				break;
		}
		if(found)
			break;
	}

	//cout << endl << "Constructing graph" << endl;
	
	for(list<int>::iterator it = y_list.begin(); it != y_list.end(); ++it)
	{
		//cout << (*it) << ": ";
		for(list<int>::iterator it2 = dicto[*it].begin(); it2 != dicto[*it].end(); ++it2)
		{
			//cout << *it2 << ' ';
			p1 = myPoint(*it2, *it);
			
			list<int>::iterator it3 = it2;
			++it3;
			
			for(; it3 != dicto[*it].end(); ++it3)
			{
				p2 = myPoint(*it3, *(it));
				if(chessDistance(p1, p2) == 1)
				{
					float curr_dist = 1;
					if(manhattanDistance(p1, p2) == 2)
						curr_dist = 1.4;
					if( !((src_pt == p1 && dst_pt == p2) || (src_pt == p2 && dst_pt == p1)))
						addEdge(graph, pointToVal[p1],pointToVal[p2], curr_dist);
				}
			}
			
			list<int>::iterator it_temp = y_list.end();
			--it_temp;
			if(it != it_temp)
			{
				list<int>::iterator it2_temp = it;
				++it2_temp;
				for(it3 = dicto[*(it2_temp)].begin(); it3 != dicto[*(it2_temp)].end(); ++it3)
				{
					p2 = myPoint(*it3, *(it2_temp));
					if(chessDistance(p1, p2) == 1)
					{
						float curr_dist = 1;
						if(manhattanDistance(p1, p2) == 2)
							curr_dist = 1.4;
						if( !((src_pt == p1 && dst_pt == p2) || (src_pt == p2 && dst_pt == p1)))
							addEdge(graph, pointToVal[p1],pointToVal[p2], curr_dist);
					}	
				}
			}
		}
		//cout << endl;
	}

	return graph;
}
