// C / C++ program for Dijkstra's shortest path algorithm for adjacency
// list representation of graph

#if !defined(DIJKSTRA_H)
#define DIJKSTRA_H

#include <stdio.h>
#include "opencvScript.h"
#include <stdlib.h>
#include <limits.h>
#define INF 1000000.0 
#include <ctime>

// A structure to represent a node in adjacency list
struct AdjListNode
{
    int dest;
    float weight;
    struct AdjListNode* next;
};
 
// A structure to represent an adjacency liat
struct AdjList
{
    struct AdjListNode *head;  // pointer to head node of list
    void display(map<int, myPoint> valToPoint)
    {
		struct AdjListNode *temp = this->head;
		cout << "Dest: ";
		while(temp != NULL)
		{
			cout << "(" << valToPoint[temp -> dest].x << ", " << valToPoint[temp -> dest].y << ") ";
			temp = temp->next;
		}
		cout << endl;
    }
};
 
// A structure to represent a graph. A graph is an array of adjacency lists.
// Size of array will be V (number of vertices in graph)
struct Graph
{
    int V;
    struct AdjList* array;
    void display(map<int, myPoint> valToPoint)
    {
    	cout<<"Number of vertices: " << V << endl;
    	for(int i=0; i<V; i++)
    	{
    		cout << "Src: " << "(" << valToPoint[i].x << ", " << valToPoint[i].y << ") ";
    		array[i].display(valToPoint);
    	}
    }
};

// Structure to represent a min heap node 
struct MinHeapNode
{
    int  v;
    float dist;
};

// Structure to represent a min heap
struct MinHeap
{
    int size;      // Number of heap nodes present currently
    int capacity;  // Capacity of min heap
    int *pos;     // This is needed for decreaseKey()
    struct MinHeapNode **array;
};
 
// A utility function to create a new adjacency list node
struct AdjListNode* newAdjListNode(int dest, float weight);
struct Graph* createGraph(int V);
void addEdge(struct Graph* graph, int src, int dest, float weight);
// A utility function to create a new Min Heap Node
struct MinHeapNode* newMinHeapNode(int v, int dist);
// A utility function to create a Min Heap
struct MinHeap* createMinHeap(int capacity);
// A utility function to swap two nodes of min heap. Needed for min heapify
void swapMinHeapNode(struct MinHeapNode** a, struct MinHeapNode** b);
// A standard function to heapify at given idx
// This function also updates position of nodes when they are swapped.
// Position is needed for decreaseKey()
void minHeapify(struct MinHeap* minHeap, int idx);
// A utility function to check if the given minHeap is ampty or not
int isEmpty(struct MinHeap* minHeap);
// Standard function to extract minimum node from heap
struct MinHeapNode* extractMin(struct MinHeap* minHeap);
// Function to decreasy dist value of a given vertex v. This function
// uses pos[] of min heap to get the current index of node in min heap
void decreaseKey(struct MinHeap* minHeap, int v, float dist);
// A utility function to check if a given vertex
// 'v' is in min heap or not
bool isInMinHeap(struct MinHeap *minHeap, int v);
// A utility function used to print the solution
void printArr(float dist[], int n, map<int, myPoint> valToPoint, vector<int> prev, int src, int dest);
// The main function that calulates distances of shortest paths from src to all
// vertices. It is a O(ELogV) function
void dijkstra(struct Graph* graph, int src, int dest, map<int, myPoint> valToPoint);
// Assigns the points to a graph structure and returns the graph
Graph* assignToGraph(vector<myPoint> points, map<int,list<int> > dicto, list<int> y_list, map<int, myPoint> &valToPoint, map<myPoint, int> &pointToVal, myPoint &src_pt, myPoint &dst_pt);

#endif
