function [erro,repeat,corresp, match_score,matches, twi,twoutFull,wout,feat1,feat2,strength1,strength2,cornerness1, cornerness2]=repeatability(file1,file2,Hom,imf1,imf2,common_part,fidOut,sz)
%
%
%Computes repeatability and overlap score between two lists of features
%detected in two images.
%   [erro,repeat,corresp,matched,matchedp]=repeatability('file1','file2','H4to1','imf1','imf2','-ko');
%
%IN:
%    file1 - file 1 with detected features
%    file2 - file 2 with detected features
%    H - file with 3x3 Homography matrix from image 1 to 2, x2=H*x1
%        Assumes that image coordiantes are 0..width.
%    imf1 - image file  1
%    imf2 - image file  2
%    common_part - flag should be set to 1 for repeatability and 0 for descriptor performance
%
%OUT :    erro - overlap %
%         repeat - repeatability %
%         corresp - nb of correspondences
%         match_score  - matching score
%         matches - number of correct nearest neighbour matches
%         twi - matrix with overlap errors<50\%
%
%  region file format :
%--------------------
%descriptor_size
%nbr_of_regions
%x1 y1 a1 b1 c1 d1 d2 d3 ...
%x2 y2 a2 b2 c2 d1 d2 d3 ...
%....
%....
%---------------------
%x, y - center coordinates
%a, b, c - ellipse parameters ax^2+2bxy+cy^2=1
%d1 d2 d3 ... - descriptor invariants
%if descriptor_size<=1 the descriptor is ignored

[~,repVar] = globalVariables;

%inputs that can possibly made global later
ovErrResolutionNscaling=0;
if ~exist('iscaling','var') || ~exist('iresolution','var')
    iscaling=1;
    iresolution=50;
end
singleScaleRep=0;
repMinOvThresh=70;
%%%
% Default values for descriptor matching - we do not run this test right
% now
matches = 0;
match_score = 0;
twi = 0;

% these are parameters for the overlap checking algorithm


if ~exist('fidOut','var')
    fidOut = 1;
end

fprintf(fidOut,'Reading and sorting the regions...\n');

[f1 s1 dimdesc1]=loadFeatures(file1);
f1=unique(f1','rows'); f1=f1'; s1=size(f1,2);
if size(f1,1) > 5
   strength1 = f1(6,:); 
   cornerness1 = f1(7,:); 
else
    strength1 = zeros(1,size(f1,2));
    cornerness1  = zeros(1,size(f1,2));
end
f1=abs(f1(1:5,:));

[f2 s2 dimdesc2]=loadFeatures(file2);
f2=unique(f2','rows'); f2=f2'; s2=size(f2,2);
if size(f2,1) > 5
   strength2 = f2(6,:); 
   cornerness2 = f2(7,:); 
else
    strength2 = zeros(1,size(f2,2));
    cornerness2  = zeros(1,size(f2,2));
end
f2=abs(f2(1:5,:));

H=  load(Hom);


fprintf(fidOut,'nb of regions in file1 %d - descriptor dimension %d.\n',s1,dimdesc1);
fprintf(fidOut,'nb of regions in file2 %d - descriptor dimension %d.\n',s2,dimdesc2);


if size(f1,1)==5 & size(f1,1)==size(f2,1)
    fprintf(fidOut,'%s looks like file with affine regions...\n',file1);
    if  size(f1,1)~= 5 | size(f1,1) ~= 5
        error('Wrong ascii format of %s or %s files.',file1,file2);
    end
elseif dimdesc1>1 & dimdesc1==dimdesc2
    fprintf(fidOut,'%s, %s look like files with descriptors...\n',file1,file2);
else
    error('Different descriptor dimension in %s or %s files.',file1,file2);
end

dimdesc=dimdesc1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% allocate vectors for the features
feat1=zeros(9+dimdesc,s1);
feat2=zeros(9+dimdesc,s2);
feat1(1:5,1:s1)=f1(1:5,1:s1);
feat2(1:5,1:s2)=f2(1:5,1:s2);
if size(f1,1)>1
    feat1(10:9+dimdesc,1:s1)=f1(6:5+dimdesc,1:s1);
    feat2(10:9+dimdesc,1:s2)=f2(6:5+dimdesc,1:s2);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%project regions from one image to the other
HI=H(:, 1:3);
H=inv(HI);
fprintf(fidOut,'Projecting 1 to 2...');
[feat1 feat1t scales1]=project_regions(feat1',HI);
fprintf(fidOut,'and 2 to 1...\n');
[feat2 feat2t scales2]=project_regions(feat2',H);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if common_part==1
    fprintf(fidOut,'Removing features from outside of the common image part...\n');
    im1=imread(imf1);
    im2=imread(imf2);
    im1x=size(im1);
    im1y=im1x(1);
    im1x=im1x(2);
    im2x=size(im2);
    im2y=im2x(1);
    im2x=im2x(2);
    ind=find((feat1(:,1)+feat1(:,8))<im1x & (feat1(:,1)-feat1(:,8))>0 & (feat1(:,2)+feat1(:,9))<im1y & (feat1(:,2)-feat1(:,9))>0);
    ind1A = ind;
    feat1=feat1(ind,:);
    feat1t=feat1t(ind,:);
    strength1 = strength1(ind);
    cornerness1 = cornerness1(ind);

    ind=find((feat1t(:,1)+feat1t(:,8))<im2x & (feat1t(:,1)-feat1t(:,8))>0 & (feat1t(:,2)+feat1t(:,9))<im2y & (feat1t(:,2)-feat1t(:,9))>0);
    ind1B = ind;
    feat1=feat1(ind,:);
    feat1t=feat1t(ind,:);
    strength1 = strength1(ind);
    cornerness1 = cornerness1(ind);    
    scales1=scales1(ind);
    
    ind=find((feat2(:,1)+feat2(:,8))<im2x & (feat2(:,1)-feat2(:,8))>0 & (feat2(:,2)+feat2(:,9))<im2y & (feat2(:,2)-feat2(:,9))>0);
    ind2A = ind;
    feat2t=feat2t(ind,:);
    feat2=feat2(ind,:);
    strength2 = strength2(ind);
    cornerness2 = cornerness2(ind);
    
    ind=find((feat2t(:,1)+feat2t(:,8))<im1x & (feat2t(:,1)-feat2t(:,8))>0 & (feat2t(:,2)+feat2t(:,9))<im1y & (feat2t(:,2)-feat2t(:,9))>0);
    ind2B = ind;
    feat2t=feat2t(ind,:);
    feat2=feat2(ind,:);
    strength2 = strength2(ind);
    cornerness2 = cornerness2(ind);
    scales2=scales2(ind);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    fprintf(fidOut,'nb of regions in common part in image1 %d.\n',size(feat1,1));
    fprintf(fidOut,'nb of regions in common part in image2 %d.\n',size(feat2t,1));
end

sf=min([size(feat1,1) size(feat2t,1)]);

feat1=feat1';
feat1t=feat1t';
feat2t=feat2t';
feat2=feat2';


fprintf(fidOut,'Computing overlap error & selecting one-to-one correspondences: ');

if repVar.optimizeRepeatabilityMemory
    tic;
    
    numFeat1 = size(feat1,2);
    numFeat2 = size(feat2t,2);
    
    twoutFull = zeros(numFeat2,numFeat1);
    %     doutFull = [];
    
    for featIdx=1:numFeat1
        feat1Part = feat1(:,featIdx);
        
        if ovErrResolutionNscaling
            [~,twoutFull(:,featIdx),~,~,~,~,tdout]=ellipse_overlap5(feat1Part,feat2t,iscaling,iresolution);
        else
            % c_eoverlap is a C implementation to compute the overlap error.
            [~, twoutFull(:,featIdx), ~, tdout]=c_eoverlap(feat1Part,feat2t,common_part);
        end
        
        %         twoutFull = [twoutFull twout];
        %         doutFull = [doutFull dout];
        if ~mod(featIdx,500)
            fprintf('iter: %d\n',featIdx);
        end
    end
    
    t=toc;
    fprintf(fidOut,' %.1f sec.\n',t);
    
    if repVar.optimizedRepCorresp
        
        [numFeat1 numFeat2] = size(twoutFull);
        woutFull = 100*ones([numFeat1 numFeat2]);
        
        [mini minj] = find(twoutFull<repMinOvThresh);
        
        numPoints = size(minj,1);
        minvalUnsorted = zeros(numPoints,1);
        
        for idx=1:numPoints
            minvalUnsorted(idx) = twoutFull(mini(idx),minj(idx));
        end
        [minval loc] = sort(minvalUnsorted);
        
        testj = [];
        testi = [];
        
        for idx = 1:numPoints
            if isempty(find(testj == minj(loc(idx)))) && isempty(find(testi == mini(loc(idx))))
                woutFull(mini(loc(idx)),minj(loc(idx))) = minval(idx);
                testj = [testj; minj(loc(idx))];
                testi = [testi; mini(loc(idx))];
            end
            if ~mod(idx,10000)
                disp(['iter: ' num2str(idx)]);
            end
        end
        
    else
        woutFull=repOut(twoutFull);
    end
    
    %Checking
    % [wout, twout, dout, tdout]=c_eoverlap(feat1t,feat2,common_part);
    % aaa=woutFull-wout;
    % find(aaa>0)
    
    wout = woutFull;
    %     dout = doutFull;
    %     twout = twoutFull;
else
    tic;
    
    % c_eoverlap is a C implementation to compute the overlap error.
    if ovErrResolutionNscaling
        [wout,twout,faarea,fadr,fabna,dout,tdout]=ellipse_overlap5(feat1,feat2t,iscaling,iresolution);
    else
        [wout, twout, dout, tdout]=c_eoverlap(feat1,feat2t,common_part);
    end
    
    t=toc;
    fprintf(fidOut,' %.1f sec.\n',t);
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

erro=10:10:60;
corresp=zeros(1,6);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute the number of correspondences
for i=1:6,
    wi=(wout<erro(i));
    corresp(i)=sum(sum(wi));
end

if singleScaleRep
    AnalyseRepOfPts(wout,feat1,feat2,imf1,imf2,ind2A,ind2B,ind1A,ind1B,sz);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
repeat=100*corresp/sf;


fprintf(fidOut,'\noverlap error: ');
fprintf(fidOut,'%.1f ',erro);
fprintf(fidOut,'\nrepeatability: ');
fprintf(fidOut,'%.1f ',repeat);
fprintf(fidOut,'\nnb of correspondences: ');
fprintf(fidOut,'%.0f ',corresp);
fprintf(fidOut,'\n');

% Descriptor Matching Score calculation code - we don't use yet
%
% match_overlap=40;
% if common_part==0
%     match_overlap=50;
% end
%
% fprintf(fidOut,'Matching with the descriptor for the overlap error < %d%%\n',match_overlap);
%
% if common_part==0
%     twi=(twout<match_overlap);
% else
%     twi=(wout<match_overlap);
% end
%
% dx=(dout<10000).*(twi);
% matches=sum(sum(dx));
% match_score=100*matches/sf;
%
% fprintf(fidOut,'\nMatching score  %0.1f, nb of correct matches %.1f.\n',match_score,matches);



%end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [feat,featp,scales]=project_regions(feat,H)

s=size(feat);
s1=s(1);

featp=feat;
scales=zeros(1,s1);

for c1=1:s1,%%%%%%%%%%%%%%%%%
    %feat(c1,3:5)=(1/25)*feat(c1,3:5);
    Mi1=[feat(c1,3) feat(c1,4);feat(c1,4) feat(c1,5)];
    
    %compute affine transformation
    try
    [v1 e1]=eig(Mi1);
    catch
        ss=1;
    end;
    d1=(1/sqrt(e1(1)));
    d2=(1/sqrt(e1(4)));
    sc1=sqrt(d1*d2);
    feat(c1,6)=d1;
    feat(c1,7)=d2;
    scales(c1)=sqrt(feat(c1,6)*feat(c1,7));
    
    %bounding box
    feat(c1,8) = sqrt(feat(c1,5)/(feat(c1,3)*feat(c1,5) - feat(c1,4)^2));
    feat(c1,9) = sqrt(feat(c1,3)/(feat(c1,3)*feat(c1,5) - feat(c1,4)^2));
    
    
    Aff=getAff(feat(c1,1),feat(c1,2),sc1, H);
    
    %project to image 2
    l1=[feat(c1,1),feat(c1,2),1];
    l1_2=H*l1';
    l1_2=l1_2/l1_2(3);
    featp(c1,1)=l1_2(1);
    featp(c1,2)=l1_2(2);
    BMB=inv(Aff*inv(Mi1)*Aff');
    [v1 e1]=eig(BMB);
    featp(c1,6)=(1/sqrt(e1(1)));
    featp(c1,7)=(1/sqrt(e1(4)));
    featp(c1,3:5)=[BMB(1) BMB(2) BMB(4)];
    %bounding box in image 2
    featp(c1,8) = sqrt(featp(c1,5)/(featp(c1,3)*featp(c1,5) - featp(c1,4)^2));
    featp(c1,9) = sqrt(featp(c1,3)/(featp(c1,3)*featp(c1,5) - featp(c1,4)^2));
end
%end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Aff=getAff(x,y,sc,H)

h11=H(1);
h12=H(4);
h13=H(7);
h21=H(2);
h22=H(5);
h23=H(8);
h31=H(3);
h32=H(6);
h33=H(9);
fxdx=h11/(h31*x + h32*y +h33) - (h11*x + h12*y +h13)*h31/(h31*x + h32*y +h33)^2;
fxdy=h12/(h31*x + h32*y +h33) - (h11*x + h12*y +h13)*h32/(h31*x + h32*y +h33)^2;

fydx=h21/(h31*x + h32*y +h33) - (h21*x + h22*y +h23)*h31/(h31*x + h32*y +h33)^2;
fydy=h22/(h31*x + h32*y +h33) - (h21*x + h22*y +h23)*h32/(h31*x + h32*y +h33)^2;

Aff=[fxdx fxdy;fydx fydy];
%end




